@@ -13,8 +13,8 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -30,7 +30,7 @@ extern char *h_errlist[];
 
 /* Return a string describing the host lookup error code HERR.  The returned
    value may be in a static buffer (and in any case shouldn't be written to).  */
-char *
+const char *
 hstrerror (herr)
   int herr;
 {
