@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -134,6 +134,7 @@ protected:
    int max_retries;
 
    bool	 mkdir_p;
+   bool	 rename_f;
 
    int	 saved_errno;
 
@@ -158,6 +159,7 @@ protected:
    int chmod_mode;
    bool ascii;
    bool norest_manual;
+   bool fragile;
 
    int	priority;   // higher priority can take over other session.
    int	last_priority;
@@ -168,7 +170,12 @@ protected:
    void Fatal(const char *mess);
    xstring error;
    int error_code;
+
    xstring_c location;
+   xstring_c location_file;
+   int location_mode;
+   bool location_permanent;
+
    xstring_c suggested_filename;
    void SetSuggestedFileName(const char *fn);
 
@@ -202,16 +209,16 @@ public:
    const char  *GetUser() const { return user; }
    const char  *GetPassword() const { return pass; }
    const char  *GetPort() const { return portname; }
-   const char  *GetConnectURL(int flags=0) const;
-   const char  *GetFileURL(const char *file,int flags=0) const;
+   const xstring& GetConnectURL(int flags=0) const;
+   const xstring& GetFileURL(const char *file,int flags=0) const;
    enum { NO_PATH=1,WITH_PASSWORD=2,NO_PASSWORD=4,NO_USER=8 };
    const char *GetLastDisconnectCause() const { return last_disconnect_cause; }
 
    void Connect(const char *h,const char *p);
    void ConnectVerify();
    void PathVerify(const Path &);
-   virtual void AnonymousLogin();
    virtual void Login(const char *u,const char *p);
+   void AnonymousLogin() { Login(0,0); }
 
    // reset location-related state on Connect/Login/AnonymousLogin
    virtual void ResetLocationData();
@@ -227,7 +234,7 @@ public:
    void AsciiTransfer() { ascii=true; }
    virtual void Close();
 
-   void Rename(const char *rfile,const char *to) { Open2(rfile,to,RENAME); }
+   void Rename(const char *rfile,const char *to,bool clobber=false);
    void Link(const char *f1,const char *f2) { Open2(f1,f2,LINK); }
    void Symlink(const char *f1,const char *f2) { Open2(f1,f2,SYMLINK); }
    void Mkdir(const char *rfile,bool allpath=false);
@@ -252,6 +259,7 @@ public:
    off_t GetRealPos() const { return real_pos<0?pos:real_pos; }
    void SeekReal() { pos=GetRealPos(); }
    void RereadManual() { norest_manual=true; }
+   void SetFragile() { fragile=true; }
 
    const Path& GetCwd() const { return cwd; }
    const Path& GetNewCwd() const { return *new_cwd; }
@@ -296,7 +304,8 @@ public:
       STORE_FAILED,
       LOGIN_FAILED,
       DO_AGAIN,
-      NOT_SUPP
+      NOT_SUPP,
+      FRAGILE_FAILED,
    };
 
    virtual const char *StrError(int err);
@@ -311,7 +320,12 @@ public:
 
    static bool NotSerious(int err) { return temporary_network_error(err); }
 
-   const char *GetNewLocation() { return location; }
+   const char *GetNewLocation() const { return location; }
+   const char *GetNewLocationFile() const { return location_file; }
+   int GetNewLocationMode() const { return location_mode; }
+   bool IsNewLocationPermanent() const { return location_permanent; }
+   virtual FileAccess *GetNewLocationFA() const;
+
    const char *GetSuggestedFileName() { return suggested_filename; }
    const char *GetEntityContentType() { return entity_content_type; }
    const char *GetEntityCharset() { return entity_charset; }
@@ -322,7 +336,7 @@ public:
    typedef FileAccess *SessionCreator();
    class Protocol
    {
-      static xmap<Protocol*> proto_by_name;
+      static xmap_p<Protocol> proto_by_name;
       const char *proto;
       SessionCreator *New;
 
@@ -330,6 +344,7 @@ public:
    public:
       static FileAccess *NewSession(const char *proto);
       Protocol(const char *proto,SessionCreator *creator);
+      static void ClassCleanup() { proto_by_name.empty(); }
    };
 
    static void Register(const char *proto,SessionCreator *creator)
@@ -467,12 +482,15 @@ class ListInfo : public FileAccessOperation
 protected:
    FileAccess::Path saved_cwd;
    Ref<FileSet> result;
+   Ref<FileSet> excluded;
 
    const char *exclude_prefix;
    const PatternSet *exclude;
 
    unsigned need;
    bool follow_symlinks;
+   bool try_recursive;
+   bool is_recursive;
 
    void PrepareToDie();
    ~ListInfo();
@@ -480,10 +498,13 @@ protected:
 public:
    ListInfo(FileAccess *session,const char *path);
 
-   void SetExclude(const char *p,const PatternSet *e) { exclude_prefix=p; exclude=e; }
+   void SetExclude(const char *p,const PatternSet *e) { exclude_prefix=p; exclude=e; excluded=new FileSet(); }
+   void TryRecursive(bool y=true) { try_recursive=y; }
 
    // caller has to delete the resulting FileSet itself.
    FileSet *GetResult() { return result.borrow(); }
+   FileSet *GetExcluded() { return excluded.borrow(); }
+   bool IsRecursive() const { return is_recursive; }
 
    void Need(unsigned mask) { need|=mask; }
    void NoNeed(unsigned mask) { need&=~mask; }
