@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,14 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: GetJob.h,v 1.18 2010/04/29 11:00:42 lav Exp $ */
-
 #ifndef GETJOB_H
 #define GETJOB_H
 
 #include "CopyJob.h"
 
-class GetJob : public CopyJobEnv
+class GetJob : public CopyJobEnv, ResClient
 {
    FileCopyPeer *SrcLocal(const char *src);
    FileCopyPeer *DstLocal(const char *dst);
@@ -37,23 +35,17 @@ class GetJob : public CopyJobEnv
 protected:
    FileCopyPeer *CreateCopyPeer(FileAccess *session,const char *path,FA::open_mode mode);
    FileCopyPeer *CreateCopyPeer(const FileAccessRef& session,const char *path,FA::open_mode mode);
-   void	 NextFile();
+   void	NextFile();
 
+   bool make_dirs;
    bool delete_files;
    bool remove_target_first;
    bool truncate_target_first;
-   xstring backup_file;
-   mode_t file_mode;
-   Ref<FDStream> local;
    bool reverse;
 
-   void RemoveBackupFile();
-
 public:
    GetJob(FileAccess *s,ArgV *a,bool c=false);
 
-   int	 Do();
-
    void DeleteFiles() { delete_files=true; }
    void RemoveSourceLater() { delete_files=true; }
    void Reverse() { reverse=true; } // put
