@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -259,6 +259,7 @@ bool file_info::validate()
 }
 
 
+#undef debug
 #define debug(str) Log::global->Format(10,"* %s\n",str)
 
 static bool try_apache_listing(file_info &info,const char *str)
@@ -276,6 +277,22 @@ static bool try_apache_listing(file_info &info,const char *str)
    }
    return false;
 }
+static bool try_apache_listing_iso(file_info &info,const char *str)
+{
+   info.clear();
+
+   // apache listing with ISO time: YYYY-MM-DD hh:mm size
+   int n=sscanf(str,"%4d-%2d-%2d %2d:%2d %30s",
+	       &info.year,&info.month,&info.day,
+	       &info.hour,&info.minute,info.size_str);
+   if(n==6 && (info.size_str[0]=='-' || is_ascii_digit(info.size_str[0])))
+   {
+      debug("apache listing matched (ISO time)");
+      info.month--;
+      return true;
+   }
+   return false;
+}
 static bool try_apache_listing_unusual(file_info &info,const char *str)
 {
    info.clear();
@@ -579,6 +596,7 @@ static bool try_csm_proxy(file_info &info,const char *str)
 	      Log::global->Format(10,
 		      "* try_csm_proxy: unknown file type '%s'\n",
 		      additional_file_info);
+	      return false;
 	  }
       }
    }
@@ -703,15 +721,17 @@ static int parse_html(const char *buf,int buf_len,bool eof,const Ref<Buffer>& li
 	 { "overlay", "src" },
 	 { "applet", "code" },
 	 { "script", "src" },
+	 { "source", "src" },
 	 { "embed", "src" },
 	 { "bgsound", "src" },
 	 { "area", "href" },
 	 { "img", "lowsrc" },
 	 { "input", "src" },
 	 { "layer", "src" },
-	 { "table", "background"},
-	 { "th", "background"},
-	 { "td", "background"},
+	 { "table", "background" },
+	 { "th", "background" },
+	 { "td", "background" },
+	 { "link", "href" },
 	 /* Tags below this line are treated specially.  */
 	 { "base", "href" },
 	 { "meta", "content" },
@@ -743,6 +763,14 @@ static int parse_html(const char *buf,int buf_len,bool eof,const Ref<Buffer>& li
 
    decode_amps(link_target);  // decode all &amp; and similar
 
+   // inherit the protocol if omitted
+   if(link_target.begins_with("//") && prefix && prefix->proto) {
+      xstring& new_link=xstring::get_tmp("")
+	 .append_url_encoded(prefix->proto,URL_UNSAFE,0)
+	 .append(':').append(link_target);
+      link_target.swap(new_link);
+   }
+
    if(hftp)
    {
       // workaround proxy bugs.
@@ -932,10 +960,12 @@ parse_url_again:
       show_in_list=false;  // makes apache listings look better.
 
    skip_len=tag_len;
+
    // try to find file info
    {
       const char *more1;
-      char *str,*str_with_tags, *str2;
+      xstring str, str_with_tags;
+      char  *str2;
       xstring line_add;
       xstring info_string;
       int type;
@@ -984,13 +1014,12 @@ parse_url_again:
 	 more1+=2;
 
       // the buffer is not null-terminated, so we need this
-      str=string_alloca(eol-more1);
-      memcpy(str,more1+1,eol-more1-1);
-      str[eol-more1-1]=0;
-      str_with_tags=alloca_strdup(str);
-      remove_tags(str);
+      str.nset(more1 + 1, eol - more1 - 1);
+      str_with_tags.nset(more1 + 1, eol - more1 - 1);
+      str.set_length(remove_tags(str.get_non_const()));
 
       if(try_apache_listing(info,str)		&& info.validate()) goto got_info;
+      if(try_apache_listing_iso(info,str)	&& info.validate()) goto got_info;
       if(try_apache_listing_unusual(info,str)	&& info.validate()) goto got_info;
       if(try_netscape_proxy(info,str)		&& info.validate()) goto got_info;
       if(try_squid_eplf(info,str) && info.validate())
@@ -999,14 +1028,14 @@ parse_url_again:
 	 skip_len=eol-buf+eol_len;
 	 goto got_info;
       }
-      if(try_lighttpd_listing(info,str_with_tags) && info.validate())
+      if(try_lighttpd_listing(info,str_with_tags.get_non_const()) && info.validate())
 	 goto got_info;
       if(try_mini_proxy(info,str)		&& info.validate()) goto got_info;
       if(try_apache_unixlike(info,str,more,more1,info_string)
       && info.validate())
 	 goto got_info;
       if(try_roxen(info,str)			&& info.validate()) goto got_info;
-      if(try_squid_ftp(info,str,str_with_tags) && info.validate())
+      if(try_squid_ftp(info,str,str_with_tags.get_non_const()) && info.validate())
       {
 	 // skip rest of line, because there may be href to link target.
 	 skip_len=eol-buf+eol_len;
@@ -1183,7 +1212,6 @@ info_done:
 
       set->Add(fi);
    }
-
    return skip_len;
 }
 
@@ -1404,7 +1432,9 @@ FileSet *Http::ParseLongList(const char *b,int len,int *err) const
    xstring_c base_href;
    for(;;)
    {
-      int n=parse_html(b,len,true,Ref<Buffer>::null,set,0,&prefix,&base_href);
+       int clen = len;
+       if(clen > 1000)clen = 1000;
+      int n=parse_html(b,clen,true,Ref<Buffer>::null,set,0,&prefix,&base_href);
       if(n==0)
 	 break;
       b+=n;
