@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -63,10 +63,13 @@ private:
 
    long long bytes_transferred;
    long long bytes_to_transfer;
+
    Ref<FileSet> target_set;
+   Ref<FileSet> target_set_excluded;
    Ref<FileSet> source_set;
    Ref<FileSet> target_set_recursive;
    Ref<FileSet> source_set_recursive;
+
    Ref<FileSet> to_transfer;
    Ref<FileSet> to_mkdir;
    Ref<FileSet> same;
@@ -75,8 +78,9 @@ private:
    Ref<FileSet> old_files_set;
    Ref<FileSet> new_files_set;
    Ref<FileSet> to_rm_src;
-   void	 InitSets(const FileSet *src,const FileSet *dst);
-   bool only_dirs;
+   void InitSets(); // deduce above sets from source_set and target_set
+   void ExcludeEmptyDir(const char *target_rel_dir);
+   bool only_dirs;  // to_transfer (or to_mkdir) contains directories only
 
    void RemoveSourceLater(const FileInfo *fi) {
       if(!remove_source_files)
@@ -101,6 +105,7 @@ private:
 
    bool create_target_dir;
    bool	no_target_dir;	   // target directory does not exist (for script_only)
+   bool remove_this_source_dir;
 
    SMTaskRef<ListInfo> source_list_info;
    SMTaskRef<ListInfo> target_list_info;
@@ -121,19 +126,22 @@ private:
       Statistics();
       void Reset();
       void Add(const Statistics &);
-      bool HaveSomethingDone(int);
+      bool HaveSomethingDone(unsigned mirror_flags);
    };
    Statistics stats;
 
    double transfer_time_elapsed;
    TimeDate transfer_start_ts;
-   int	 root_transfer_count;
-   int	 &transfer_count;
 
-   int	 flags;
+   /* root_transfer_count is the global counter in the root mirror,
+    * and weight of a non-root mirror in global transfer_count otherwise. */
+   int	 root_transfer_count;
+
+   unsigned flags;
    recursion_mode_t recursion_mode;
    int	 max_error_count;
 
+   Ref<PatternSet> top_exclude;
    Ref<PatternSet> my_exclude;
    const PatternSet *exclude;
 
@@ -173,23 +181,17 @@ private:
    void HandleChdir(FileAccessRef& session, int &redirections);
    void HandleListInfoCreation(const FileAccessRef& session,SMTaskRef<ListInfo>& list_info,
 	    const char *relative_dir);
-   void HandleListInfo(SMTaskRef<ListInfo>& list_info,Ref<FileSet>& set);
+   void HandleListInfo(SMTaskRef<ListInfo>& list_info,Ref<FileSet>& set,Ref<FileSet> *fsx=0);
 
+   void MirrorStarted();
+   void MirrorFinished();
    void TransferStarted(class CopyJob *cp);
    void JobStarted(Job *j);
    void TransferFinished(Job *j);
    void JobFinished(Job *j);
 
-   off_t GetBytesCount() {
-      long long bytes_count=Job::GetBytesCount();
-      if(!parent_mirror) {
-	 // bytes_transferred is cumulative over the mirror tree,
-	 // add it on the top only
-	 bytes_count+=bytes_transferred;
-      }
-      return bytes_count;
-   }
-   double GetTimeSpent() { return transfer_time_elapsed+(transfer_count>0?now-root_mirror->transfer_start_ts:0.); }
+   off_t GetBytesCount();
+   double GetTimeSpent();
 
 public:
    enum
@@ -217,17 +219,20 @@ public:
       OVERWRITE=1<<20,
       UPLOAD_OLDER=1<<21,
       TRANSFER_ALL=1<<22,
+      TARGET_FLAT=1<<23,
+      DELETE_EXCLUDED=1<<24,
+      REVERSE=1<<25,
    };
-   void SetFlags(int f,bool v)
+   void SetFlags(unsigned f,bool v)
    {
       if(v)
 	 flags|=f;
       else
 	 flags&=~f;
    }
-   bool FlagsSet(int f)	   { return (flags&f)==f; }
-   bool FlagSet(int f)	   { return (flags&f); }
-   bool AnyFlagSet(int f)  { return (flags&f); }
+   bool FlagsSet(unsigned f)   const { return (flags&f)==f; }
+   bool FlagSet(unsigned f)    const { return (flags&f); }
+   bool AnyFlagSet(unsigned f) const { return (flags&f); }
 
    MirrorJob(MirrorJob *parent,FileAccess *f,FileAccess *target,
       const char *new_source_dir,const char *new_target_dir);
@@ -245,6 +250,7 @@ public:
    void	 SetExclude(const PatternSet *x) { exclude=x; }
    void	 SetSizeRange(Range *r) { my_size_range=r; size_range=my_size_range; }
    void	 SetSizeRange(const Range *r) { size_range=r; }
+   void	 SetTopExclude(PatternSet *x) { top_exclude=x; }
 
    void	 SetVerbose(int v) { verbose_report=v; }
 
@@ -274,6 +280,11 @@ public:
    void SetOnChange(const char *oc);
    static const char *AddPattern(Ref<PatternSet>& exclude,char opt,const char *optarg);
    static const char *AddPatternsFrom(Ref<PatternSet>& exclude,char opt,const char *file);
+
+   FileAccess const& GetExecSession()
+   {
+      return FlagSet(REVERSE)?*target_session:*source_session;
+   }
 };
 
 #endif//MIRRORJOB_H
