@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -149,14 +149,11 @@ int SFtp::Do()
       const char *prog=Query("connect-program",hostname);
       if(!prog || !prog[0])
 	 prog="ssh -a -x";
-      char *a=alloca_strdup(prog);
-      ArgV *cmd=new ArgV;
-      for(a=strtok(a," "); a; a=strtok(0," "))
-	 cmd->Add(a);
+      ArgV args;
       if(!strchr(init,'/'))
       {
 	 if(init[0])
-	    cmd->Add("-s");   // run ssh2 subsystem
+	    args.Add("-s");   // run ssh2 subsystem
 	 // sftpd does not have a greeting
 	 received_greeting=true;
       }
@@ -164,20 +161,21 @@ int SFtp::Do()
 	 init=xstring::cat("echo SFTP: >&2;",init,NULL);
       if(user)
       {
-	 cmd->Add("-l");
-	 cmd->Add(user);
+	 args.Add("-l");
+	 args.Add(user);
       }
       if(portname)
       {
-	 cmd->Add("-p");
-	 cmd->Add(portname);
+	 args.Add("-p");
+	 args.Add(portname);
       }
-      cmd->Add(hostname);
+      args.Add(hostname);
       if(init[0])
-	 cmd->Add(init);
-      xstring_ca cmd_str(cmd->Combine(0));
+	 args.Add(init);
+      xstring_ca cmd_q(args.CombineShellQuoted(0));
+      xstring& cmd_str=xstring::cat(prog," ",cmd_q.get(),NULL);
       LogNote(9,"%s (%s)",_("Running connect program"),cmd_str.get());
-      ssh=new PtyShell(cmd);
+      ssh=new PtyShell(cmd_str);
       ssh->UsePipes();
       state=CONNECTING;
       timeout_timer.Reset();
@@ -263,13 +261,14 @@ int SFtp::Do()
       if(s==0)
       {
 	 // no more data, set attributes and close the file.
-	 if(entity_date!=NO_DATE)
-	 {
-	    Request_FSETSTAT *req=new Request_FSETSTAT(handle,protocol_version);
+	 Request_FSETSTAT *req=new Request_FSETSTAT(handle,protocol_version);
+	 if(entity_date!=NO_DATE) {
 	    req->attrs.mtime=entity_date;
 	    req->attrs.flags|=SSH_FILEXFER_ATTR_MODIFYTIME;
-	    SendRequest(req,Expect::IGNORE);
 	 }
+	 req->attrs.size=pos;
+	 req->attrs.flags|=SSH_FILEXFER_ATTR_SIZE;
+	 SendRequest(req,Expect::IGNORE);
 	 CloseHandle(Expect::DEFAULT);
 	 state=WAITING;
 	 m=MOVED;
@@ -300,12 +299,7 @@ void SFtp::MoveConnectionHere(SFtp *o)
    recv_translate=o->recv_translate.borrow();
    send_translate=o->send_translate.borrow();
    rate_limit=o->rate_limit.borrow();
-   expect_queue_size=o->expect_queue_size; o->expect_queue_size=0;
-   expect_chain=o->expect_chain; o->expect_chain=0;
-   expect_chain_end=o->expect_chain_end;
-   if(expect_chain_end==&o->expect_chain)
-      expect_chain_end=&expect_chain;
-   o->expect_chain_end=&o->expect_chain;
+   expect_queue.move_here(o->expect_queue);
    timeout_timer.Reset(o->timeout_timer);
    ssh_id=o->ssh_id;
    state=CONNECTED;
@@ -341,10 +335,6 @@ void SFtp::Init()
    eof=false;
    received_greeting=false;
    password_sent=0;
-   expect_queue_size=0;
-   expect_chain=0;
-   expect_chain_end=&expect_chain;
-   ooo_chain=0;
    protocol_version=0;
    send_translate=0;
    recv_translate=0;
@@ -456,7 +446,7 @@ SFtp::unpack_status_t SFtp::UnpackPacket(Buffer *b,SFtp::Packet **p)
    if(res!=UNPACK_SUCCESS)
       return res;
 
-   Log::global->Format(9,"<--- got a packet, length=%d, type=%d(%s), id=%u\n",
+   LogRecvF(9,"got a packet, length=%d, type=%d(%s), id=%u\n",
       probe.GetLength(),probe.GetPacketType(),probe.GetPacketTypeText(),probe.GetID());
 
    switch(probe.GetPacketType())
@@ -534,7 +524,7 @@ void SFtp::SendRequest(Packet *request,Expect::expect_t tag,int i)
 {
    request->SetID(ssh_id++);
    request->ComputeLength();
-   Log::global->Format(9,"---> sending a packet, length=%d, type=%d(%s), id=%u\n",
+   LogSendF(9,"sending a packet, length=%d, type=%d(%s), id=%u\n",
       request->GetLength(),request->GetPacketType(),request->GetPacketTypeText(),request->GetID());
    request->Pack(send_buf.get_non_const());
    PushExpect(new Expect(request,tag,i));
@@ -608,8 +598,16 @@ void SFtp::SendRequest()
 	 SetError(NOT_SUPP);
 	 break;
       }
+      unsigned options=0;
+      if(rename_f) {
+	 options=SSH_FXF_RENAME_OVERWRITE;
+	 if(protocol_version<5) {
+	    // overwrite is not supported, remove the target explicitly
+	    SendRequest(new Request_REMOVE(WirePath(file1)),Expect::IGNORE);
+	 }
+      }
       SendRequest(new Request_RENAME(WirePath(file),WirePath(file1),
-			SSH_FXF_RENAME_NATIVE,protocol_version),Expect::DEFAULT);
+			options,protocol_version),Expect::DEFAULT);
       state=WAITING;
       break;
    }
@@ -722,8 +720,7 @@ void SFtp::Close()
    CloseHandle(Expect::IGNORE);
    super::Close();
    // don't need these out-of-order packets anymore
-   while(ooo_chain)
-      DeleteExpect(&ooo_chain);
+   ooo_chain.truncate();
    if(recv_buf)
       recv_buf->Resume();
 }
@@ -827,7 +824,7 @@ void SFtp::HandleExpect(Expect *e)
 	    SetError(NO_FILE,strerror(ENOTDIR));
 	    break;
 	 }
-	 if(mode==CHANGE_DIR && RespQueueIsEmpty())
+	 if(mode==CHANGE_DIR && !HasExpect(Expect::CWD))
 	 {
 	    cwd.Set(file);
 	    eof=true;
@@ -849,17 +846,10 @@ void SFtp::HandleExpect(Expect *e)
 	    handle_x.appendf("%02X",(unsigned char)handle[i]);
 	 LogNote(9,"got file handle %s (%d)",handle_x.get(),handle_len);
 	 request_pos=real_pos=pos;
-	 if(mode==RETRIEVE)
+	 if(mode==RETRIEVE) {
 	    SendRequest(new Request_FSTAT(handle,
 	       SSH_FILEXFER_ATTR_SIZE|SSH_FILEXFER_ATTR_MODIFYTIME|SSH_FILEXFER_ATTR_PERMISSIONS,
 	       protocol_version),Expect::INFO);
-	 else if(mode==STORE && pos>0)
-	 {
-	    // truncate the file at write position.
-	    Request_FSETSTAT *req=new Request_FSETSTAT(handle,protocol_version);
-	    req->attrs.size=pos;
-	    req->attrs.flags|=SSH_FILEXFER_ATTR_SIZE;
-	    SendRequest(req,Expect::IGNORE);
 	 }
       }
       else
@@ -886,7 +876,7 @@ void SFtp::HandleExpect(Expect *e)
 	    d->GetData(&b,&s);
 	    LogNote(9,"data packet: pos=%lld, size=%d",(long long)r->pos,s);
 	    file_buf->Put(b,s);
-	    if(d->Eof())
+	    if(d->Eof() || eof)
 	       goto eof;
 	    if(r->len > unsigned(s))   // received less than requested?
 	    {
@@ -900,11 +890,15 @@ void SFtp::HandleExpect(Expect *e)
 	 }
 	 else
 	 {
-	    if(e->next!=ooo_chain)
-	       LogNote(9,"put a packet with id=%d on out-of-order chain (need_pos=%lld packet_pos=%lld)",
-		  reply->GetID(),(long long)(pos+file_buf->Size()),(long long)r->pos);
-	    e->next=ooo_chain;
-	    ooo_chain=e;
+	    LogNote(9,"put a packet with id=%d on out-of-order chain (need_pos=%lld packet_pos=%lld)",
+	       reply->GetID(),(long long)(pos+file_buf->Size()),(long long)r->pos);
+	    if(ooo_chain.count()>=64)
+	    {
+	       LogError(0,"Too many out-of-order packets");
+	       Disconnect();
+	       return;
+	    }
+	    ooo_chain.append(e);
 	    return;
 	 }
       }
@@ -944,7 +938,7 @@ void SFtp::HandleExpect(Expect *e)
 	       file_set->Add(info);
 	    }
 	 }
-	 if(r->Eof())
+	 if(r->Eof() || eof)
 	    goto eof;
       }
       else
@@ -954,12 +948,13 @@ void SFtp::HandleExpect(Expect *e)
 	    if(((Reply_STATUS*)reply)->GetCode()==SSH_FX_EOF)
 	    {
 	    eof:
-	       if(!eof)
-		  LogNote(9,"eof");
 	       eof=true;
 	       state=DONE;
-	       if(file_buf && !ooo_chain)
+	       if(file_buf && ooo_chain.count()==0 && !HasExpectBefore(reply->GetID(),Expect::DATA))
+	       {
+		  LogNote(9,"eof");
 		  file_buf->PutEOF();
+	       }
 	       break;
 	    }
 	 }
@@ -1006,8 +1001,10 @@ void SFtp::HandleExpect(Expect *e)
    case Expect::WRITE_STATUS:
       if(reply->TypeIs(SSH_FXP_STATUS))
       {
-	 if(((Reply_STATUS*)reply)->GetCode()==SSH_FX_OK)
+	 if(((Reply_STATUS*)reply)->GetCode()==SSH_FX_OK) {
+	    TrySuccess();
 	    break;
+	 }
       }
       SetError(NO_FILE,reply);
       break;
@@ -1030,6 +1027,7 @@ void SFtp::HandleExpect(Expect *e)
 
 void SFtp::RequestMoreData()
 {
+   Enter(this);
    if(mode==RETRIEVE) {
       int req_len=size_read;
       SendRequest(new Request_READ(handle,request_pos,req_len),Expect::DATA);
@@ -1037,6 +1035,7 @@ void SFtp::RequestMoreData()
    } else if(mode==LIST || mode==LONG_LIST) {
       SendRequest(new Request_READDIR(handle),Expect::DATA);
    }
+   Leave(this);
 }
 
 int SFtp::HandleReplies()
@@ -1051,23 +1050,24 @@ int SFtp::HandleReplies()
    if(!recv_buf)
       return MOVED;
 
-   int i=0;
-   Expect *ooo_scan=ooo_chain;
-   while(ooo_scan)
-   {
-      Expect *next=ooo_scan->next;
-      ooo_chain=next;
-      HandleExpect(ooo_scan);
-      ooo_scan=next;
-      if(++i>64)
-      {
-	 LogError(0,"Too many out-of-order packets");
-	 Disconnect();
-	 return MOVED;
+   if(file_buf) {
+      off_t need_pos=pos+file_buf->Size();
+      // there are usually a few of out-of-order packets, no need for fast search
+      for(int i=0; i<ooo_chain.count(); i++) {
+	 if(ooo_chain[i]->has_data_at_pos(need_pos)) {
+	    Expect *e=ooo_chain[i];
+	    ooo_chain[i]=0; // to keep the Expect
+	    ooo_chain.remove(i);
+	    HandleExpect(e);
+	 }
       }
    }
-   if(!ooo_chain && eof && file_buf && !file_buf->Eof())
+
+   if(eof && file_buf && !file_buf->Eof() && ooo_chain.count()==0 && !HasExpect(Expect::DATA))
+   {
+      LogNote(9,"eof");
       file_buf->PutEOF();
+   }
 
    if(recv_buf->Size()<4)
    {
@@ -1111,51 +1111,21 @@ int SFtp::HandleReplies()
    HandleExpect(e);
    return MOVED;
 }
-SFtp::Expect **SFtp::FindExpect(Packet *p)
-{
-   unsigned id=p->GetID();
-   for(Expect **scan=&expect_chain; *scan; scan=&scan[0]->next)
-   {
-      if(scan[0]->request->GetID()==id)
-      {
-	 assert(!scan[0]->reply);
-	 scan[0]->reply=p;
-	 return scan;
-      }
-   }
-   return 0;
-}
 void SFtp::PushExpect(Expect *e)
 {
-   e->next=*expect_chain_end;
-   *expect_chain_end=e;
-   expect_chain_end=&e->next;
-   expect_queue_size++;
-}
-void SFtp::DeleteExpect(Expect **e)
-{
-   if(expect_chain_end==&e[0]->next)
-      expect_chain_end=e;
-   Expect *d=*e;
-   *e=e[0]->next;
-   delete d;
-   expect_queue_size--;
+   expect_queue.add(e->request->GetKey(),e);
 }
 SFtp::Expect *SFtp::FindExpectExclusive(Packet *p)
 {
-   Expect **e=FindExpect(p);
-   if(!e || !*e)
-      return 0;
-   Expect *res=*e;
-   if(expect_chain_end==&res->next)
-      expect_chain_end=e;
-   *e=res->next;
-   expect_queue_size--;
-   return res;
+   Expect *e=expect_queue.borrow(p->GetKey());
+   if(e)
+      e->reply=p;
+   return e;
 }
+
 void SFtp::CloseExpectQueue()
 {
-   for(Expect  *e=expect_chain; e; e=e->next)
+   for(Expect *e=expect_queue.each_begin(); e; e=expect_queue.each_next())
    {
       switch(e->tag)
       {
@@ -1179,6 +1149,28 @@ void SFtp::CloseExpectQueue()
    }
 }
 
+bool SFtp::HasExpect(Expect::expect_t tag)
+{
+   for(Expect *e=expect_queue.each_begin(); e; e=expect_queue.each_next())
+      if(e->tag==tag)
+	 return true;
+   return false;
+}
+
+static bool IsBefore(unsigned id1,unsigned id2)
+{
+   // order with wrap-around
+   return id2 - id1 < id1 - id2;
+}
+
+bool SFtp::HasExpectBefore(unsigned id,Expect::expect_t tag)
+{
+   for(Expect *e=expect_queue.each_begin(); e; e=expect_queue.each_next())
+      if(e->tag==tag && IsBefore(e->request->GetID(),id))
+	 return true;
+   return false;
+}
+
 Glob *SFtp::MakeGlob(const char *pat)
 {
    return new GenericGlob(this,pat);
@@ -1194,7 +1186,7 @@ int SFtp::Read(Buffer *buf,int size)
       return error_code;
    if(mode==CLOSED)
       return 0;
-   if(state==DONE)
+   if(state==DONE && (!file_buf || (file_buf->Size()==0 && file_buf->Eof())))
       return 0;	  // eof
    if(state==FILE_RECV)
    {
@@ -1206,7 +1198,10 @@ int SFtp::Read(Buffer *buf,int size)
 	 if(entity_size<0 || request_pos<entity_size || RespQueueSize()<2)
 	    RequestMoreData();
       }
+   }
 
+   if(file_buf && file_buf->Size()>0)
+   {
       const char *buf1;
       int size1;
       file_buf->Get(&buf1,&size1);
@@ -1220,7 +1215,9 @@ int SFtp::Read(Buffer *buf,int size)
 	 return DO_AGAIN;
       if(size>size1)
 	 size=size1;
-      buf->MoveDataHere(file_buf,size);
+      size=buf->MoveDataHere(file_buf,size);
+      if(size<=0)
+	 return DO_AGAIN;
       pos+=size;
       real_pos+=size;
       rate_limit->BytesGot(size);
@@ -1236,7 +1233,9 @@ int SFtp::Write(const void *buf,int size)
       return(0);
 
    Resume();
+   Enter(this);
    Do();
+   Leave(this);
    if(Error())
       return(error_code);
 
@@ -1258,7 +1257,6 @@ int SFtp::Write(const void *buf,int size)
    if(size<=0)
       return 0;
    file_buf->Put(static_cast<const char*>(buf),size);
-   TrySuccess();
    rate_limit->BytesPut(size);
    pos+=size;
    real_pos+=size;
@@ -1376,23 +1374,6 @@ bool SFtp::SameLocationAs(const FileAccess *fa) const
    return true;
 }
 
-void SFtp::Cleanup()
-{
-   if(hostname==0)
-      return;
-
-   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
-      fo->CleanupThis();
-
-   CleanupThis();
-}
-void SFtp::CleanupThis()
-{
-   if(mode!=CLOSED)
-      return;
-   Disconnect();
-}
-
 void SFtp::Reconfig(const char *name)
 {
    super::Reconfig(name);
@@ -1499,6 +1480,7 @@ const char *SFtp::Reply_STATUS::GetCodeText()
       "Connection lost",
       "Operation not supported",
       "Invalid handle",
+      "No such path",
       "File already exists",
       "Write protect",
       "No media",
@@ -1534,7 +1516,7 @@ void SFtp::SetError(int code,const Packet *reply)
    }
    Reply_STATUS *status=(Reply_STATUS*)reply;
    const char *message=status->GetMessage();
-   if(message)
+   if(message && *message)
    {
       SetError(code,utf8_to_lc(message));
       return;
@@ -2056,8 +2038,6 @@ FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
 
    if(!name || !name[0])
       return 0;
-   if(strchr(name,'/'))
-      return 0;
    if(name[0]=='~')
       name=dir_file(".",name);
    Ref<FileInfo> fi(new FileInfo(name));
@@ -2292,6 +2272,7 @@ int SFtpListInfo::Do()
    if(result && session->OpenMode()!=FA::ARRAY_INFO)
    {
       ubuf=0;
+      result->ExcludeCompound();
       result->rewind();
       for(FileInfo *file=result->curr(); file!=0; file=result->next())
       {
@@ -2300,6 +2281,7 @@ int SFtpListInfo::Do()
 	 {
 	    if(file->filetype==file->SYMLINK && follow_symlinks)
 	    {
+	       file->filetype=file->UNKNOWN;
 	       file->defined &= ~(file->SIZE|file->DATE|file->SYMLINK_DEF|file->MODE|file->TYPE|file->USER|file->GROUP);
 	       file->Need(file->SIZE|file->DATE|file->MODE|file->TYPE|file->USER|file->GROUP);
 	    }
