@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,18 +17,15 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Speedometer.cc,v 1.26 2009/08/18 05:25:23 lav Exp $ */
-
 #include <config.h>
-#include <math.h>
+#include <cmath>
 #include <stdlib.h>
 #include "Speedometer.h"
 #include "misc.h"
 #include "xstring.h"
 #include "ResMgr.h"
 
-#undef super
-#define super SMTask
+#define now SMTask::now
 
 Speedometer::Speedometer(const char *p)
 {
@@ -45,14 +42,10 @@ bool Speedometer::Valid()
 {
    return now>=start+TimeDiff(1,0) && now<last_bytes+TimeDiff(period,0);
 }
-int Speedometer::Do()
-{
-   TimeoutS(1);
-   return STALL;
-}
 float Speedometer::Get()
 {
    Add(0);
+   SMTask::current->Timeout(500);
    return rate;
 }
 void Speedometer::Add(int b)
@@ -87,6 +80,7 @@ void Speedometer::Add(int b)
    if(rate<0)
       rate=0;
 }
+// These `incorrect' units are used to fit on status line
 xstring& Speedometer::GetStr(float r)
 {
    if(r<1)
@@ -95,12 +89,23 @@ xstring& Speedometer::GetStr(float r)
       // for translator: those are the units. This is 'byte per second'
       return xstring::format(_("%.0fb/s"),r);
    else if(r<1024*1024)
-      // for translator: This is 'kilobyte per second'
+      // for translator: This is 'Kibibyte per second'
       return xstring::format(_("%.1fK/s"),r/1024.);
    else
-      // for translator: This is 'Megabyte per second'
+      // for translator: This is 'Mebibyte per second'
       return xstring::format(_("%.2fM/s"),r/(1024.*1024));
 }
+xstring& Speedometer::GetStrProper(float r)
+{
+   if(r<1)
+      return xstring::get_tmp("");
+   if(r<1024)
+      return xstring::format(_("%.0f B/s"),r);
+   else if(r<1024*1024)
+      return xstring::format(_("%.1f KiB/s"),r/1024.);
+   else
+      return xstring::format(_("%.2f MiB/s"),r/(1024.*1024));
+}
 xstring& Speedometer::GetStr()
 {
    return Valid() ? GetStr(Get()) : xstring::get_tmp("");
