@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,72 +29,60 @@
 
 int IOBufferSSL::Do()
 {
+   int m=STALL;
+
    if(Done() || Error())
-      return STALL;
+      return m;
 
-   int res=0;
-
-   switch(mode)
+   if(mode==PUT && Size()==0)
    {
-   case PUT:
-      if(Size()==0 && ssl->handshake_done)
-	 return STALL;
-      res=Put_LL(buffer+buffer_ptr,Size());
-      if(res>0)
+      // nothing to write, but may need to do handshake
+      if(!ssl->handshake_done)
       {
-	 buffer_ptr+=res;
-	 event_time=now;
-	 if(eof)
-	    PutEOF_LL();
-	 return MOVED;
+	 if(Put_LL("",0)<0)
+	    return MOVED;
+	 if(ssl->handshake_done && eof)
+	    ssl->shutdown();
       }
-      break;
-
-   case GET:
-      if(eof)
-	 return STALL;
-      res=Get_LL(GET_BUFSIZE);
-      if(res>0)
-      {
-	 EmbraceNewData(res);
-	 event_time=now;
-	 return MOVED;
-      }
-      if(eof)
-      {
-	 event_time=now;
-	 return MOVED;
-      }
-      break;
+      if(ssl->handshake_done && !eof)
+	 return m;
    }
-   if(res<0)
+   else
    {
-      event_time=now;
-      return MOVED;
+      // cannot use want_mask before trying to read/write, since ssl can be shared
+      if(!ssl->handshake_done || eof || Ready(ssl->fd,dir_mask()))
+	 m|=super::Do();
    }
-   if(ssl->want_in())
-      Block(ssl->fd,POLLIN);
-   if(ssl->want_out())
-      Block(ssl->fd,POLLOUT);
-   return STALL;
+   Block(ssl->fd,block_mask());
+   return m;
 }
 
 int IOBufferSSL::Get_LL(int size)
 {
-   int res=ssl->read(GetSpace(size),size);
-   if(res<0)
-   {
-      if(res==ssl->RETRY)
-	 return 0;
-      else // error
+   int total=0;
+   int max_read=0;
+   char *space=GetSpace(size);
+   while(total<size-max_read) {
+      int res=ssl->read(space+total,size-total);
+      if(res<0)
       {
-	 SetError(ssl->error,ssl->fatal);
-	 return -1;
+	 if(res==ssl->RETRY) {
+	    SetNotReady(ssl->fd,want_mask());
+	    break;
+	 } else { // error
+	    SetError(ssl->error,ssl->fatal);
+	    break;
+	 }
       }
+      if(res==0) {
+	 eof=true;
+	 break;
+      }
+      total+=res;
+      if(max_read<res)
+	 max_read=res;
    }
-   if(res==0)
-      eof=true;
-   return res;
+   return total;
 }
 
 int IOBufferSSL::Put_LL(const char *buf,int size)
@@ -102,10 +90,10 @@ int IOBufferSSL::Put_LL(const char *buf,int size)
    int res=ssl->write(buf,size);
    if(res<0)
    {
-      if(res==ssl->RETRY)
+      if(res==ssl->RETRY) {
+	 SetNotReady(ssl->fd,want_mask());
 	 return 0;
-      else // error
-      {
+      } else { // error
 	 SetError(ssl->error,ssl->fatal);
 	 return -1;
       }
