@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2014 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2020 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,6 +55,7 @@
 #include "OutputJob.h"
 #include "echoJob.h"
 #include "EditJob.h"
+#include "mmvJob.h"
 
 #include "misc.h"
 #include "alias.h"
@@ -88,7 +89,7 @@ CMD(module); CMD(mrm); CMD(mv); CMD(open); CMD(pwd); CMD(queue);
 CMD(repeat); CMD(rm); CMD(scache); CMD(set); CMD(shell); CMD(sleep);
 CMD(slot); CMD(source); CMD(subsh); CMD(suspend); CMD(tasks); CMD(torrent);
 CMD(user); CMD(ver); CMD(wait); CMD(empty); CMD(notempty); CMD(true);
-CMD(false);
+CMD(false); CMD(mmv);
 
 #define HELP_IN_MODULE "m"
 #define ALIAS_FOR(cmd) cmd_##cmd,0,#cmd
@@ -106,6 +107,8 @@ CMD(false);
 # define cmd_torrent 0
 #endif
 
+enum { DEFAULT_DEBUG_LEVEL=9 };
+
 const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 {
    {"!",       cmd_shell,  N_("!<shell-command>"),
@@ -270,7 +273,9 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "If entry type cannot be determined, it will be included in the list.\n"
 	 " -f  plain files (default)\n"
 	 " -d  directories\n"
-	 " -a  all types\n")},
+	 " -a  all types\n"
+	 " --exist      return zero exit code when the patterns expand to non-empty list\n"
+	 " --not-exist  return zero exit code when the patterns expand to an empty list\n")},
    {"help",    cmd_help,   N_("help [<cmd>]"),
 	 N_("Print help for command <cmd>, or list of available commands\n")},
    {"jobs",    cmd_jobs,   "jobs [-v] [<job_no...>]",
@@ -285,12 +290,15 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 N_("`lftp' is the first command executed by lftp after rc files\n"
 	 " -f <file>           execute commands from the file and exit\n"
 	 " -c <cmd>            execute the commands and exit\n"
+	 " --norc              don't execute rc files from the home directory\n"
 	 " --help              print this help and exit\n"
 	 " --version           print lftp version and exit\n"
-	 "Other options are the same as in `open' command\n"
+	 "Other options are the same as in `open' command:\n"
 	 " -e <cmd>            execute the command just after selecting\n"
 	 " -u <user>[,<pass>]  use the user/password for authentication\n"
 	 " -p <port>           use the port for connection\n"
+	 " -s <slot>           assign the connection to this slot\n"
+	 " -d                  switch on debugging mode\n"
 	 " <site>              host name, URL or bookmark name\n")},
    {"ln",      cmd_ln,	    N_("ln [-s] <file1> <file2>"),
 	 N_("Link <file1> to <file2>\n")},
@@ -333,8 +341,11 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -O <base> specifies base directory or URL where files should be placed\n")},
    {"mrm",     cmd_mrm,    N_("mrm <files>"),
 	 N_("Removes specified files with wildcard expansion\n")},
-   {"mv",      cmd_mv,	    N_("mv <file1> <file2>"),
+   {"mv",      cmd_mv,	   N_("mv <file1> <file2>"),
 	 N_("Rename <file1> to <file2>\n")},
+   {"mmv",      cmd_mmv,   N_("mmv [OPTS] <files> <target-dir>"),
+	 N_("Move <files> to <target-directory> with wildcard expansion\n"
+	 " -O <dir>  specifies the target directory (alternative way)\n")},
    {"nlist",   cmd_ls,     N_("[re]nlist [<args>]"),
 	 N_("List remote file names.\n"
 	 "By default, nlist output is cached, to see new listing use `renlist' or\n"
@@ -345,6 +356,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -u <user>[,<pass>]  use the user/password for authentication\n"
 	 " -p <port>           use the port for connection\n"
 	 " -s <slot>           assign the connection to this slot\n"
+	 " -d                  switch on debugging mode\n"
 	 " <site>              host name, URL or bookmark name\n")},
    {"pget",    cmd_get,    N_("pget [OPTS] <rfile> [-o <lfile>]"),
 	 N_("Gets the specified file using several connections. This can speed up transfer,\n"
@@ -423,7 +435,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "exact application of the setting. See lftp(1) for details.\n"
          "If set is called with no variable then only altered settings are listed.\n"
 	 "It can be changed by options:\n"
-   	 " -a  list all settings, including default values\n"
+	 " -a  list all settings, including default values\n"
 	 " -d  list only default values, not necessary current ones\n")},
    {"shell", ALIAS_FOR2("!",shell)},
    {"site",    cmd_ls,	   N_("site <site-cmd>"),
@@ -432,7 +444,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"sleep",   cmd_sleep, 0, HELP_IN_MODULE},
    {"slot",    cmd_slot, 0,
         N_("Usage: slot [<label>]\n"
-   	"List assigned slots.\n"
+	"List assigned slots.\n"
 	"If <label> is specified, switch to the slot named <label>.\n")},
    {"source",  cmd_source, N_("source <file>"),
 	 N_("Execute commands recorded in file <file>\n")},
@@ -460,6 +472,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {".notempty",cmd_notempty,0,0},
    {".true",   cmd_true,   0,0},
    {".false",  cmd_false,  0,0},
+   {".mplist", cmd_ls,	   0,0},
 };
 const int CmdExec::static_cmd_table_length=sizeof(static_cmd_table)/sizeof(static_cmd_table[0]);
 
@@ -727,21 +740,48 @@ void CmdExec::Exit(int code)
    return;
 }
 
+void CmdExec::enable_debug(const char *opt)
+{
+   int level=DEFAULT_DEBUG_LEVEL;
+   if(opt && isdigit((unsigned char)opt[0]))
+      level=atoi(opt);
+   const char *c="debug";
+   ResMgr::Set("log:enabled",c,"yes");
+   ResMgr::Set("log:level",c,xstring::format("%d",level));
+}
+
 CmdFeeder *lftp_feeder=0;
 Job *CmdExec::builtin_lftp()
 {
    int c;
    xstring cmd;
-   bool debug=false;
+   xstring rc;
+   ArgV open("open");
+   open.Add("--lftp");
+
+   enum {
+      OPT_USER,
+      OPT_PASSWORD,
+      OPT_ENV_PASSWORD,
+   };
    static struct option lftp_options[]=
    {
       {"help",no_argument,0,'h'},
       {"version",no_argument,0,'v'},
+      {"debug",optional_argument,0,'d'},
+      {"rcfile",required_argument,0,'r'},
+      // other options are for "open" command
+      {"port",required_argument,0,'p'},
+      {"user",required_argument,0,OPT_USER},
+      {"password",required_argument,0,OPT_PASSWORD},
+      {"env-password",no_argument,0,OPT_ENV_PASSWORD},
+      {"execute",required_argument,0,'e'},
+      {"no-bookmark",no_argument,0,'B'},
+      {"slot",required_argument,0,'s'},
       {0,0,0,0}
    };
 
-   opterr=false;
-   while((c=args->getopt_long("+f:c:vhd",lftp_options))!=EOF)
+   while((c=args->getopt_long("+f:c:vhdu:p:e:s:B",lftp_options))!=EOF)
    {
       switch(c)
       {
@@ -757,36 +797,74 @@ Job *CmdExec::builtin_lftp()
 	 cmd.append(';');
 	 break;
       case('c'):
-	 cmd.set_allocated(args->CombineCmd(args->getindex()-1));
-	 cmd.append("\n\n");
+	 args->CombineCmdTo(cmd,args->getindex()-1).append("\n\n");
+	 args->seek(args->count());
 	 break;
       case('d'):
-	 debug=true;
+	 enable_debug(optarg);
 	 break;
+      case('r'):
+	 rc.append("&&source ").append_quoted(optarg);
+	 break;
+
+      // "open" command options are passed along
+      case('s'):
+	 open.Add("-s");
+	 break;
+      case('p'):
+	 open.Add("-p").Add(optarg);
+	 break;
+      case('u'):
+	 open.Add("-u").Add(optarg);
+	 break;
+      case(OPT_USER):
+	 open.Add("--user").Add(optarg);
+	 break;
+      case(OPT_PASSWORD):
+	 open.Add("--password").Add(optarg);
+	 break;
+      case(OPT_ENV_PASSWORD):
+	 open.Add("--env-password");
+	 break;
+      case('e'):
+	 open.Add("-e").Add(optarg);
+	 break;
+      case('B'):
+	 open.Add("-B");
+	 break;
+
+      case('?'):
+	 eprintf(_("Try `%s --help' for more information\n"),args->a0());
+	 return 0;
       }
    }
-   opterr=true;
 
-   if(cmd)
-   {
-      PrependCmd(cmd);
-      if(debug)
-	 PrependCmd("debug;");
-   }
+   for(const char *arg=args->getcurr(); arg; arg=args->getnext())
+      open.Add(arg);
 
-   if(Done() && lftp_feeder)  // no feeder and no commands
+   // feeder should be set before PrependCmd
+   if(!cmd && lftp_feeder)  // no feeder and no commands
    {
       SetCmdFeeder(lftp_feeder);
       lftp_feeder=0;
       FeedCmd("||command exit\n");   // if the command fails, quit
    }
 
-   if(!cmd)
-   {
-      /* if no lftp-specific options were found, call open */
-      args->rewind();
-      return builtin_open();
+   // prepended commands are executed in reverse order: rc, cmd, open
+   if(open.count()>2) {
+      if(cmd) {
+	 eprintf(_("%s: -c, -f, -v, -h conflict with other `open' options and arguments\n"),args->a0());
+	 return 0;
+      }
+      xstring_ca open_cmd(open.CombineQuoted());
+      PrependCmd(open_cmd);
    }
+
+   if(cmd)
+      PrependCmd(cmd);
+   if(rc)
+      PrependCmd(rc);
+
    exit_code=0;
    return 0;
 }
@@ -795,7 +873,6 @@ Job *CmdExec::builtin_open()
 {
    ReuseSavedSession();
 
-   bool	 debug=false;
    const char *port=NULL;
    const char *host=NULL;
    const char *path=NULL;
@@ -812,6 +889,7 @@ Job *CmdExec::builtin_open()
       OPT_USER,
       OPT_PASSWORD,
       OPT_ENV_PASSWORD,
+      OPT_LFTP,
    };
    static struct option open_options[]=
    {
@@ -823,16 +901,16 @@ Job *CmdExec::builtin_open()
       {"debug",optional_argument,0,'d'},
       {"no-bookmark",no_argument,0,'B'},
       {"slot",required_argument,0,'s'},
-      {"help",no_argument,0,'h'},
+      {"lftp",no_argument,0,OPT_LFTP},
       {0,0,0,0}
    };
 
-   while((c=args->getopt_long("u:p:e:s:dBh",open_options))!=EOF)
+   while((c=args->getopt_long("u:p:e:s:dB",open_options))!=EOF)
    {
       switch(c)
       {
       case('s'):
-      	if (*optarg) ChangeSlot(optarg);
+	if (*optarg) ChangeSlot(optarg);
         break;
       case('p'):
 	 port=optarg;
@@ -846,7 +924,7 @@ Job *CmdExec::builtin_open()
 	 if(sep==NULL)
 	    sep=strchr(optarg,':');
 	 if(sep==NULL)
-   	    break;
+	    break;
 	 *sep=0;
 	 pass=sep+1;
 	 insecure=true;
@@ -862,7 +940,7 @@ Job *CmdExec::builtin_open()
 	 pass=getenv("LFTP_PASSWORD");
 	 break;
       case('d'):
-	 debug=true;
+	 enable_debug(optarg);
 	 break;
       case('e'):
 	 cmd_to_exec=optarg;
@@ -870,18 +948,11 @@ Job *CmdExec::builtin_open()
       case('B'):
 	 no_bm=true;
 	 break;
-      case('h'):
-	 if(!strcmp(op,"lftp"))
-	 {
-	    PrependCmd("help lftp");
-	    return 0;
-	 }
+      case(OPT_LFTP):
+	 op="lftp";
+	 break;
       case('?'):
-	 if(!strcmp(op,"lftp"))
-	    eprintf(_("Try `%s --help' for more information\n"),op);
-	 else
-	    eprintf(_("Usage: %s [-e cmd] [-p port] [-u user[,pass]] <host|url>\n"),
-	       op);
+	 eprintf(_("Usage: %s [-e cmd] [-p port] [-u user[,pass]] <host|url>\n"),op);
 	 return 0;
       }
    }
@@ -901,13 +972,9 @@ Job *CmdExec::builtin_open()
       xstring& cmd=xstring::get_tmp("open -B ");
       if(user)
       {
-	 cmd.append("-u ");
-	 cmd.append_quoted(user);
+	 cmd.append("--user ").append_quoted(user);
 	 if(pass)
-	 {
-	    cmd.append(",");
-	    cmd.append_quoted(pass);
-	 }
+	    cmd.append(" --password ").append_quoted(pass);
 	 cmd.append(' ');
       }
       if(port)
@@ -974,7 +1041,7 @@ Job *CmdExec::builtin_open()
 	    FileAccess *new_session=FileAccess::New(uc.proto,host,port);
 	    if(!new_session)
 	    {
-	       eprintf("%s: %s%s\n",args->a0(),uc.proto.get(),
+	       eprintf("%s: %s%s\n",op,uc.proto.get(),
 			_(" - not supported protocol"));
 	       return 0;
 	    }
@@ -1004,7 +1071,7 @@ Job *CmdExec::builtin_open()
       }
       if(host && host[0] && session->GetHostName()==0)
 	 session->Connect(host,port);
-      if(user)
+      if(user && *session->GetProto())
       {
 	 if(!pass)
 	    pass=GetPass(_("Password: "));
@@ -1063,9 +1130,6 @@ Job *CmdExec::builtin_open()
       PrependCmd(s);
    }
 
-   if(debug)
-      PrependCmd("debug\n");
-
    if(slot)
       ConnectionSlot::Set(slot,session);
 
@@ -1092,6 +1156,7 @@ Job *CmdExec::builtin_glob()
    int opt;
    GlobURL::type_select glob_type=GlobURL::FILES_ONLY;
    const char *cmd=0;
+   bool nullglob=false;
 
    static struct option glob_options[]=
    {
@@ -1115,9 +1180,11 @@ Job *CmdExec::builtin_glob()
 	 break;
       case('e'):
 	 cmd=".notempty";
+	 nullglob=true;
 	 break;
       case('E'):
 	 cmd=".empty";
+	 nullglob=true;
 	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),op);
@@ -1131,6 +1198,7 @@ Job *CmdExec::builtin_glob()
    if(args->count()<2)
    {
       eprintf(_("Usage: %s [OPTS] command args...\n"),op);
+      RevertToSavedSession();
       return 0;
    }
    assert(args_glob==0 && glob==0);
@@ -1142,10 +1210,12 @@ Job *CmdExec::builtin_glob()
    {
       args_glob=0;
       args->rewind();
+      RevertToSavedSession();
       return cmd_command(this);
    }
    glob=new GlobURL(session,pat,glob_type);
-   RevertToSavedSession();
+   if(nullglob)
+      glob->NullGlob();
    builtin=BUILTIN_GLOB;
    return this;
 }
@@ -1356,6 +1426,10 @@ CMD(ls)
       if(!strcmp(op,"site"))
 	 args->insarg(1,"SITE");
    }
+   else if(!strcmp(op,".mplist")) {
+      nlist=true;
+      mode=FA::MP_LIST;
+   }
 
    xstring_ca a(args->Combine(nlist?1:0));
 
@@ -1639,52 +1713,64 @@ CMD(cat)
 
 CMD(get)
 {
+   static struct option get_options[] = {
+      {"continue",no_argument,0,'c'},
+      {"Remove-source-files",no_argument,0,'E'},
+      {"remove-target",no_argument,0,'e'},
+      {"ascii",no_argument,0,'a'},
+      {"target-directory",required_argument,0,'O'},
+      {"destination-directory",required_argument,0,'O'},
+      {"quiet",no_argument,0,'q'},
+      {"parallel",optional_argument,0,'P'},
+      {"use-pget-n",optional_argument,0,'n'},
+      {"glob",no_argument,0,256+'g'},
+      {"reverse",no_argument,0,256+'R'},
+      {0}
+   };
+   const char *opts="+cEeaO:qP";
+
    int opt;
    bool cont=false;
-   const char *opts="+cEeuaO:";
    const char *op=args->a0();
-   ArgV	 *get_args=new ArgV(op);
+   Ref<ArgV> get_args(new ArgV(op));
    int n_conn=1;
+   int parallel=0;
    bool del=false;
    bool del_target=false;
    bool ascii=false;
    bool glob=false;
    bool make_dirs=false;
    bool reverse=false;
-   xstring_c output_dir;
+   bool quiet=false;
+   const char *output_dir=0;
 
    if(!strncmp(op,"re",2))
    {
       cont=true;
-      opts="+EuaO:";
+      opts="+EaO:qP:";
    }
    if(!strcmp(op,"pget"))
    {
-      opts="+n:ceuO:";
+      opts="+n:ceO:q";
       n_conn=0; // default, which means to take pget:default-n
    }
    else if(!strcmp(op,"put") || !strcmp(op,"reput"))
    {
       reverse=true;
    }
-   else if(!strcmp(op,"mget"))
+   else if(!strcmp(op,"mget") || !strcmp(op,"mput"))
    {
       glob=true;
-      opts="cEeadO:";
-   }
-   else if(!strcmp(op,"mput"))
-   {
-      glob=true;
-      opts="cEeadO:";
-      reverse=true;
+      opts="cEeadO:qP:";
+      reverse=(op[1]=='p');
    }
    if(!reverse)
    {
       const char *od=ResMgr::Query("xfer:destination-directory",session->GetHostName());
       if(od && *od)
-	 output_dir.set(od);
+	 output_dir=od;
    }
-   while((opt=args->getopt(opts))!=EOF)
+   while((opt=args->getopt_long(opts,get_options))!=EOF)
    {
       switch(opt)
       {
@@ -1692,12 +1778,15 @@ CMD(get)
 	 cont=true;
 	 break;
       case('n'):
-	 if(!isdigit((unsigned char)optarg[0]))
-	 {
-	    eprintf(_("%s: -n: Number expected. "),op);
-	    goto err;
-	 }
-	 n_conn=atoi(optarg);
+	 if(optarg) {
+	    if(!isdigit((unsigned char)optarg[0]))
+	    {
+	       eprintf(_("%s: %s: Number expected. "),"-n",op);
+	       goto err;
+	    }
+	    n_conn=atoi(optarg);
+	 } else
+	    n_conn=3;
 	 break;
       case('E'):
 	 del=true;
@@ -1712,15 +1801,39 @@ CMD(get)
 	 make_dirs=true;
 	 break;
       case('O'):
-	 output_dir.set(optarg);
+	 output_dir=optarg;
+	 break;
+      case('q'):
+	 quiet=true;
+	 break;
+      case('P'):
+	 if(optarg) {
+	    if(!isdigit((unsigned char)optarg[0]))
+	    {
+	       eprintf(_("%s: %s: Number expected. "),"-P",op);
+	       goto err;
+	    }
+	    parallel=atoi(optarg);
+	 } else
+	    parallel=3;
+	 break;
+      case(256+'R'):
+	 reverse=!reverse;
+	 break;
+      case(256+'g'):
+	 glob=true;
 	 break;
       case('?'):
       err:
 	 eprintf(_("Try `help %s' for more information.\n"),op);
-	 delete get_args;
 	 return 0;
       }
    }
+   if(cont && del_target) {
+      eprintf(_("%s: --continue conflicts with --remove-target.\n"),op);
+      return 0;
+   }
+   JobRef<GetJob> j;
    if(glob)
    {
       if(args->getcurr()==0)
@@ -1730,54 +1843,49 @@ CMD(get)
 	 eprintf(_("File name missed. "));
 	 goto err;
       }
-      delete get_args;
-      // remove options
-      while(args->getindex()>1)
-	 args->delarg(1);
-      mgetJob *j=new mgetJob(session->Clone(),args.borrow(),cont,make_dirs);
-      if(reverse)
-	 j->Reverse();
-      if(del)
-	 j->DeleteFiles();
-      if(ascii)
-	 j->Ascii();
+      mgetJob *mj=new mgetJob(session->Clone(),args,cont,make_dirs);
       if(output_dir)
-	 j->OutputDir(output_dir.borrow());
-      return j;
+	 mj->OutputDir(output_dir);
+      j=mj;
    }
-   args->back();
-   const char *a=args->getnext();
-   if(a==0)
-      goto file_name_missed;
-   while(a)
+   else
    {
-      const char *src=a;
-      const char *dst=0;
-      a=args->getnext();
-      if(a && !strcmp(a,"-o"))
+      args->back();
+      const char *a=args->getnext();
+      if(a==0)
+	 goto file_name_missed;
+      while(a)
       {
-	 dst=args->getnext();
+	 const char *src=a;
+	 const char *dst=0;
 	 a=args->getnext();
+	 if(a && !strcmp(a,"-o"))
+	 {
+	    dst=args->getnext();
+	    a=args->getnext();
+	 }
+	 if(reverse)
+	    src=expand_home_relative(src);
+	 dst=output_file_name(src,dst,!reverse,output_dir,false);
+	 get_args->Append(src);
+	 get_args->Append(dst);
       }
-      if(reverse)
-	 src=expand_home_relative(src);
-      dst=output_file_name(src,dst,!reverse,output_dir,false);
-      get_args->Append(src);
-      get_args->Append(dst);
+      j=new GetJob(session->Clone(),get_args.borrow(),cont);
    }
-
-   GetJob *j=new GetJob(session->Clone(),get_args,cont);
+   if(reverse)
+      j->Reverse();
    if(del)
       j->DeleteFiles();
    if(del_target)
       j->RemoveTargetFirst();
    if(ascii)
       j->Ascii();
-   if(reverse)
-      j->Reverse();
    if(n_conn!=1)
       j->SetCopyJobCreator(new pCopyJobCreator(n_conn));
-   return j;
+   if(parallel>0)
+      j->SetParallel(parallel);
+   j->Quiet(quiet);
+   return j.borrow();
 }
 
 CMD(edit)
@@ -2046,30 +2154,24 @@ CMD(exit)
 CMD(debug)
 {
    const char *op=args->a0();
-   int	 new_dlevel=9;
-   char	 *debug_file_name=0;
-   int 	 fd=-1;
+   int	 new_dlevel=DEFAULT_DEBUG_LEVEL;
+   const char *debug_file_name=0;
    bool  enabled=true;
    bool	 show_pid=false;
    bool	 show_time=false;
    bool	 show_context=false;
+   int	 trunc=0;
 
    int opt;
-   while((opt=args->getopt("o:ptc"))!=EOF)
+   while((opt=args->getopt("To:ptc"))!=EOF)
    {
       switch(opt)
       {
+      case('T'):
+	 trunc=O_TRUNC;
+	 break;
       case('o'):
 	 debug_file_name=optarg;
-	 if(fd!=-1)
-	    close(fd);
-	 fd=open(debug_file_name,O_WRONLY|O_CREAT|O_APPEND|O_NONBLOCK,0600);
-	 if(fd==-1)
-	 {
-	    perror(debug_file_name);
-	    return 0;
-	 }
-	 fcntl(fd,F_SETFD,FD_CLOEXEC);
 	 break;
       case 'p':
 	 show_pid=true;
@@ -2086,11 +2188,6 @@ CMD(debug)
       }
    }
 
-   if(fd==-1)
-      Log::global->SetOutput(2,false);
-   else
-      Log::global->SetOutput(fd,true);
-
    const char *a=args->getcurr();
    if(a)
    {
@@ -2107,17 +2204,20 @@ CMD(debug)
       }
    }
 
-   if(enabled)
-   {
-      Log::global->Enable();
-      Log::global->SetLevel(new_dlevel);
-   }
-   else
-      Log::global->Disable();
+   if(debug_file_name && trunc)
+      if(truncate(debug_file_name,0) < 0)
+         fprintf(stderr, "truncate failed: %s\n", strerror(errno));
 
-   Log::global->ShowPID(show_pid);
-   Log::global->ShowTime(show_time);
-   Log::global->ShowContext(show_context);
+   const char *c="debug";
+   ResMgr::Set("log:file",c,debug_file_name?debug_file_name:"");
+
+   ResMgr::Set("log:enabled",c,enabled?"yes":"no");
+   if(enabled)
+      ResMgr::Set("log:level",c,xstring::format("%d",new_dlevel));
+
+   ResMgr::Set("log:show-pid",c,show_pid?"yes":"no");
+   ResMgr::Set("log:show-time",c,show_time?"yes":"no");
+   ResMgr::Set("log:show-ctx",c,show_context?"yes":"no");
 
 #if 0
    if(interactive)
@@ -2245,7 +2345,7 @@ CMD(kill)
       {
 	 eprintf(_("%s: %s - not a number\n"),op,arg);
 	 exit_code=1;
-      	 continue;
+	 continue;
       }
       n=atoi(arg);
       if(Job::Running(n))
@@ -2283,9 +2383,7 @@ CMD(set)
    }
    args->back();
    const char *ac=args->getnext();
-   char *a=alloca_strdup(ac);
-
-   if(a==0)
+   if(ac==0)
    {
       xstring_ca s(ResMgr::Format(with_defaults,only_defaults));
       OutputJob *out=new OutputJob(output.borrow(), args->a0());
@@ -2293,6 +2391,7 @@ CMD(set)
       return j;
    }
 
+   char *a=alloca_strdup(ac);
    char *sl=strchr(a,'/');
    char *closure=0;
    if(sl)
@@ -2413,13 +2512,59 @@ CMD(subsh)
    return e;
 }
 
+CMD(mmv)
+{
+   static const struct option mmv_opts[]=
+   {
+      {"target-directory",required_argument,0,'O'},
+      {"destination-directory",required_argument,0,'O'},
+      {"remove-target-first",no_argument,0,'e'},
+      {0}
+   };
+
+   bool remove_target=false;
+   const char *target_dir=0;
+   args->rewind();
+   int opt;
+   while((opt=args->getopt_long("eO:t:",mmv_opts,0))!=EOF)
+   {
+      switch(opt)
+      {
+      case('e'):
+	 remove_target=true;
+	 break;
+      case('t'):
+      case('O'):
+	 target_dir=optarg;
+	 break;
+      case('?'):
+      help:
+	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
+	 return 0;
+      }
+   }
+   if(!target_dir && args->count()>=3) {
+      target_dir=args->getarg(args->count()-1);
+      target_dir=alloca_strdup(target_dir);
+      args->delarg(args->count()-1);
+   }
+   if(!target_dir || args->getindex()>=args->count()) {
+      eprintf(_("Usage: %s [OPTS] <files> <target-dir>\n"),args->a0());
+      goto help;
+   }
+   mmvJob *j=new mmvJob(session->Clone(),args,target_dir,FA::RENAME);
+   if(remove_target)
+      j->RemoveTargetFirst();
+   return j;
+}
+
 CMD(mv)
 {
-   if(args->count()!=3)
+   if(args->count()!=3
+   || (args->count()==3 && last_char(args->getarg(2))=='/'))
    {
-      // xgettext:c-format
-      eprintf(_("Usage: mv <file1> <file2>\n"));
-      return 0;
+      args->setarg(0,"mmv");
+      return cmd_mmv(parent);
    }
    Job *j=new mvJob(session->Clone(),args->getarg(1),args->getarg(2));
    return j;
@@ -2656,7 +2801,7 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2014);
+      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2020);
    printf("\n");
    printf(
       _("LFTP is free software: you can redistribute it and/or modify\n"
@@ -2689,9 +2834,9 @@ CMD(ver)
    {
       const char *lib_name;
       const char *symbol;
-      enum type_t { STRING_PTR, FUNC0, INT8_8 } type;
+      enum type_t { STRING_OR_PTR, STRING_PTR, FUNC0, INT8_8 } type;
       const char *skip_prefix;
-      typedef const char *(*func0)(int);
+      typedef const char *(*func0)(void *);
       const char *query() const
 	 {
 	    int v;
@@ -2701,11 +2846,16 @@ CMD(ver)
 	    const char *str=0;
 	    switch(type)
 	    {
+	    case STRING_OR_PTR:
+	       str=(const char*)sym_ptr;
+	       if(skip_prefix && !strncmp(str,skip_prefix,sizeof(char*)))
+		  break;
+	       // FALLTHROUGH
 	    case STRING_PTR:
 	       str=*(const char**)sym_ptr;
 	       break;
 	    case FUNC0:
-	       str=((func0)sym_ptr)(0);
+	       str=((func0)sym_ptr)(NULL);
 	       break;
 	    case INT8_8:
 	       v=*(int*)sym_ptr;
@@ -2720,11 +2870,12 @@ CMD(ver)
    }
    static const libs[]=
    {
-      {"Readline",   "rl_library_version",   VersionInfo::STRING_PTR,0},
       {"Expat",	     "XML_ExpatVersion",     VersionInfo::FUNC0,     "expat_"},
-      {"OpenSSL",    "SSL_version_str",	     VersionInfo::STRING_PTR,"OpenSSL "},
       {"GnuTLS",     "gnutls_check_version", VersionInfo::FUNC0,     0},
+      {"idn2",	     "idn2_check_version",   VersionInfo::FUNC0,     0},
       {"libiconv",   "_libiconv_version",    VersionInfo::INT8_8,    0},
+      {"OpenSSL",    "SSL_version_str",      VersionInfo::STRING_OR_PTR,"OpenSSL "},
+      {"Readline",   "rl_library_version",   VersionInfo::STRING_PTR,0},
       {"zlib",	     "zlibVersion",	     VersionInfo::FUNC0,     0},
       {0}
    };
@@ -2739,11 +2890,10 @@ CMD(ver)
       snprintf(buf,sizeof(buf),", %s %s",scan->lib_name,v);
       int skip=need_comma?0:2;
       int w=mbswidth(buf+skip,mbflags);
-      if(col+w>=width)
+      if(col+w>=width && need_comma)
       {
 	 buf[1]='\n';
-	 col=w-2+skip;
-	 skip/=2;
+	 col=w-2;
       }
       else
 	 col+=w;
@@ -2830,10 +2980,7 @@ CMD(bookmark)
 	 {
 	    value=session->GetConnectURL(flags);
 	    // encode some more characters, special to CmdExec parser.
-	    xstring& a=url::encode(value,"&;|\"'\\");
-	    if(value[0] && last_char(value)!='/')
-	       a.append('/');
-	    value=a;
+	    value=url::encode(value,"&;|\"'\\");
 	 }
 	 if(value==0 || value[0]==0)
 	    value="\"\"";
@@ -2843,7 +2990,7 @@ CMD(bookmark)
 	    return 0;
 	 }
 	 lftp_bookmarks.Add(key,value);
-   	 exit_code=0;
+	 exit_code=0;
       }
    }
    else if(!strcasecmp(op,"delete"))
@@ -2863,12 +3010,13 @@ CMD(bookmark)
    {
       lftp_bookmarks.Remove(""); // force bookmark file creation
 
-      xstring cmd0("exec ${EDITOR:-vi} ");
-      cmd0.append(shell_encode(lftp_bookmarks.GetFilePath()));
-      xstring cmd1("/bin/sh -c ");
-      cmd1.append(shell_encode(cmd0));
-
-      parent->PrependCmd(xstring::get_tmp("shell ").append_quoted(cmd1));
+      const char *bin=getenv("EDITOR");
+      if (bin==NULL)
+         bin="vi";
+      xstring cmd(bin);
+      cmd.append(" ");
+      cmd.append(shell_encode(lftp_bookmarks.GetFilePath()));
+      parent->PrependCmd(xstring::get_tmp("shell ").append_quoted(cmd));
    }
    else if(!strcasecmp(op,"import"))
    {
@@ -2897,7 +3045,7 @@ CMD(bookmark)
 CMD(echo)
 {
    xstring s;
-   s.set_allocated(args->Combine(1));
+   args->CombineTo(s,1);
    if(args->count()>1 && !strcmp(args->getarg(1),"-n"))
    {
       if(s.length()<=3)
@@ -2999,7 +3147,7 @@ CMD(du)
    bool print_totals=false;
    bool all_files=false;
    bool file_count=false;
-   const char *exclude=0;
+   Ref<PatternSet> exclude;
    int human_opts=0;
 
    exit_code=1;
@@ -3062,7 +3210,9 @@ CMD(du)
 	 }
 	 break;
       case OPT_EXCLUDE:
-	 exclude=optarg;
+	 if(!exclude)
+	    exclude=new PatternSet();
+	 exclude->Add(PatternSet::EXCLUDE,new PatternSet::Glob(optarg));
 	 break;
       case '?':
       default:
@@ -3112,11 +3262,7 @@ CMD(du)
    if(separate_dirs && maxdepth != -1)
       j->set_maxdepth(maxdepth);
    if(exclude)
-   {
-      PatternSet *p=new PatternSet();
-      p->Add(p->EXCLUDE,new PatternSet::Glob(exclude));
-      j->SetExclude(p);
-   }
+      j->SetExclude(exclude.borrow());
    return j;
 }
 
@@ -3268,6 +3414,8 @@ CMD(get1)
       {"output",required_argument,0,'o'},
       {"remove-source-later",no_argument,0,'E'},
       {"remove-target-first",no_argument,0,'e'},
+      {"make-target-dir",no_argument,0,'d'},
+      {"quiet",no_argument,0,'q'},
       {0,0,0,0}
    };
    int opt;
@@ -3275,11 +3423,13 @@ CMD(get1)
    const char *dst=0;
    bool cont=false;
    bool ascii=false;
+   bool quiet=false;
+   bool do_mkdir=false;
    long long source_region_begin=0,source_region_end=FILE_END;
    long long target_region_begin=0,target_region_end=FILE_END;
    int n,p;
 
-   while((opt=args->getopt_long("arco:",get1_options))!=EOF)
+   while((opt=args->getopt_long("arco:d",get1_options))!=EOF)
    {
       switch(opt)
       {
@@ -3310,6 +3460,12 @@ CMD(get1)
 	    goto usage;
 	 }
 	 break;
+      case('q'):
+	 quiet=true;
+	 break;
+      case('d'):
+	 do_mkdir=true;
+	 break;
       case '?':
       usage:
 	 eprintf(_("Usage: %s [OPTS] file\n"),args->a0());
@@ -3362,6 +3518,8 @@ CMD(get1)
       }
    }
 
+   dst=alloca_strdup(dst); // save tmp xstring
+
    FileCopyPeer *src_peer=0;
    FileCopyPeer *dst_peer=0;
 
@@ -3376,13 +3534,17 @@ CMD(get1)
    dst_peer->AutoRename(auto_rename && ResMgr::QueryBool("xfer:auto-rename",0));
    if(!cont && (target_region_begin>0 || target_region_end!=FILE_END))
       dst_peer->SetRange(target_region_begin,target_region_end);
+   if(do_mkdir)
+      dst_peer->MakeTargetDir();
 
    FileCopy *c=FileCopy::New(src_peer,dst_peer,cont);
 
    if(ascii)
       c->Ascii();
 
-   return new CopyJob(c,src,args->a0());
+   CopyJob *cj=new CopyJob(c,src,args->a0());
+   cj->Quiet(quiet);
+   return cj;
 }
 
 CMD(slot)
@@ -3451,7 +3613,7 @@ CMD(eval)
    int base=optind;
    xstring cmd;
    if(!fmt)
-      cmd.set_allocated(args->Combine(optind));
+      args->CombineTo(cmd,optind);
    else
    {
       while(*fmt)
