@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -95,6 +95,7 @@ class Ftp : public NetAccess
       bool received_150;
 
       char type;  // type of transfer: 'A'scii or 'I'mage
+      char t_mode; // transfer mode: 'S'tream, 'Z'ipped
 
       bool dos_path;
       bool vms_path;
@@ -106,6 +107,8 @@ class Ftp : public NetAccess
       bool site_chmod_supported;
       bool site_utime_supported;
       bool site_utime2_supported;   // two-argument SITE UTIME
+      bool site_symlink_supported;
+      bool site_mkdir_supported;
       bool pret_supported;
       bool utf8_supported;
       bool lang_supported;
@@ -113,8 +116,13 @@ class Ftp : public NetAccess
       bool clnt_supported;
       bool host_supported;
       bool mfmt_supported;
+      bool mff_supported;
       bool epsv_supported;
       bool tvfs_supported;
+      bool mode_z_supported;
+      bool cepr_supported;
+
+      bool ssl_after_proxy;
 
       off_t last_rest;	// last successful REST position.
       off_t rest_pos;	// the number sent with REST command.
@@ -144,6 +152,7 @@ class Ftp : public NetAccess
 #endif
 
       xstring_c mlst_attr_supported;
+      xstring_c mode_z_opts_supported;
 
       Connection(const char *c);
       ~Connection();
@@ -170,6 +179,9 @@ class Ftp : public NetAccess
       void SendCmdF(const char *fmt,...) PRINTF_LIKE(2,3);
       int FlushSendQueueOneCmd(); // sends single command from send_cmd_buffer
 
+      void AddDataTranslator(DataTranslator *t);
+      void AddDataTranslation(const char *charset,bool translit);
+
       void SuspendInternal()
       {
 	 if(control_send) control_send->SuspendSlave();
@@ -183,6 +195,7 @@ class Ftp : public NetAccess
 	 if(data_iobuf)	data_iobuf->ResumeSlave();
       }
 
+      void CheckFEAT(char *reply,const char *line,bool trust);
    };
 
    Ref<Connection> conn;
@@ -197,6 +210,7 @@ class Ftp : public NetAccess
 	 READY,		// check response after connect
 	 REST,		// check response for REST
 	 TYPE,		// check response for TYPE
+	 MODE,
 	 CWD,		// check response for CWD
 	 CWD_CURR,	// check response for CWD into current directory
 	 CWD_STALE,	// check response for CWD when it's not critical
@@ -233,10 +247,11 @@ class Ftp : public NetAccess
       };
 
       expect_t check_case;
+      xstring_c cmd;
       xstring_c arg;
       Expect *next;
 
-      Expect(expect_t e,const char *a=0) : check_case(e), arg(a) {}
+      Expect(expect_t e,const char *a=0,const char *c=0) : check_case(e), cmd(c), arg(a) {}
       Expect(expect_t e,char c) : check_case(e)
 	 {
 	    char cc[2]={c,0};
@@ -279,7 +294,6 @@ class Ftp : public NetAccess
    void	 NoPassReqCheck(int);
    void	 proxy_LoginCheck(int);
    void	 proxy_NoPassReqCheck(int);
-   void	 CheckFEAT(char *reply);
    char *ExtractPWD();
    int   SendCWD(const char *path,const char *path_url,Expect::expect_t c);
    void	 CatchDATE(int);
@@ -299,6 +313,7 @@ class Ftp : public NetAccess
 
    pasv_state_t Handle_PASV();
    pasv_state_t Handle_EPSV();
+   pasv_state_t Handle_EPSV_CEPR();
 
    bool ServerSaid(const char *) const;
    bool NonError5XX(int act) const;
@@ -408,7 +423,6 @@ private:
    bool use_stat_for_list;
    bool use_mdtm;
    bool use_size;
-   bool use_pret;
    bool use_feat;
    bool use_mlsd;
 
@@ -424,6 +438,8 @@ private:
 
    int CanRead();
 
+   const char *path_to_send();
+
 protected:
    void PrepareToDie();
 
@@ -495,8 +511,6 @@ public:
    };
 
    void Reconfig(const char *name=0);
-   void Cleanup();
-   void CleanupThis();
 
    ListInfo *MakeListInfo(const char *path);
    Glob *MakeGlob(const char *pattern);
