@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,8 +28,10 @@
 #  include <openssl/err.h>
 #  include <openssl/rand.h>
 #  include <openssl/x509v3.h>
+#  include <openssl/x509_vfy.h>
 # endif
 
+#include "Ref.h"
 #include "xstring.h"
 
 class lftp_ssl_base
@@ -48,7 +50,7 @@ public:
    enum code { RETRY=-2, ERROR=-1, DONE=0 };
 
    void set_error(const char *s1,const char *s2);
-   void set_cert_error(const char *s);
+   void set_cert_error(const char *s,const xstring& fp);
 };
 
 #if USE_GNUTLS
@@ -68,8 +70,8 @@ typedef gnutls_x509_crt_fmt gnutls_x509_crt_fmt_t;
 typedef gnutls_datum gnutls_datum_t;
 #endif
 
-#include "SMTask.h"
-class lftp_ssl_gnutls_instance : public SMTask
+#include "ResMgr.h"
+class lftp_ssl_gnutls_instance : public ResClient
 {
    gnutls_x509_crl_t *crl_list;
    unsigned crl_list_size;
@@ -82,12 +84,11 @@ class lftp_ssl_gnutls_instance : public SMTask
 public:
    lftp_ssl_gnutls_instance();
    ~lftp_ssl_gnutls_instance();
-   int Do() { return STALL; }
    void Reconfig(const char *);
 };
 class lftp_ssl_gnutls : public lftp_ssl_base
 {
-   static lftp_ssl_gnutls_instance *instance;
+   static Ref<lftp_ssl_gnutls_instance> instance;
    gnutls_session_t session;
    gnutls_certificate_credentials_t cred;
    void verify_certificate_chain(const gnutls_datum_t *cert_chain,int cert_chain_length);
@@ -95,6 +96,7 @@ class lftp_ssl_gnutls : public lftp_ssl_base
    void verify_last_cert(gnutls_x509_crt_t crt);
    int do_handshake();
    bool check_fatal(int res);
+   static const xstring& get_fp(gnutls_x509_crt_t crt);
 public:
    static void global_init();
    static void global_deinit();
@@ -121,11 +123,12 @@ public:
 };
 class lftp_ssl_openssl : public lftp_ssl_base
 {
-   static lftp_ssl_openssl_instance *instance;
+   static Ref<lftp_ssl_openssl_instance> instance;
    SSL *ssl;
    bool check_fatal(int res);
    int do_handshake();
    const char *strerror();
+   static const xstring& get_fp(X509 *crt);
 public:
    static int verify_crl(X509_STORE_CTX *ctx);
    static int verify_callback(int ok,X509_STORE_CTX *ctx);
