@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,10 +23,12 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include "SMTask.h"
+#include "Ref.h"
+#include "ResMgr.h"
 
-class Log : public SMTask
+class Log : public ResClient
 {
+   const char *name;
    int output;
    bool need_close_output;
    bool tty;
@@ -48,37 +50,37 @@ class Log : public SMTask
    bool enabled;
    int level;
 
+   xstring buf;
+
+protected:
+   void SetOutput(int o,bool need_close);
+
 public:
-   static SMTaskRef<Log> global;
+   static Ref<Log> global;
 
    bool WillOutput(int l);
-   void DoWrite(const char *str);
-   void Write(int l,const char *str);
+   void DoWrite(const char *str,int len);
+   void DoWrite(const xstring &str) { DoWrite(str,str.length()); }
+   void Write(int l,const char *str,int len);
+   void Write(int l,const char *str) { Write(l,str,xstrlen(str)); }
+   void Write(int l,const xstring &str) { Write(l,str,str.length()); }
    void Format(int l,const char *fmt,...) PRINTF_LIKE(3,4);
    void vFormat(int l,const char *fmt,va_list v);
 
-   void SetLevel(int l) { level=l; }
-   void Enable()  { enabled=true;  }
-   void Disable() { enabled=false; }
-
-   void SetOutput(int o,bool need_close);
-
    void SetCB(tty_cb_t cb) { tty_cb=cb; }
 
    bool IsTTY() { return tty; }
 
-   void Init();
-   Log() { Init(); }
+   Log(const char *name);
    ~Log();
 
-   int Do();
-
-   void ShowPID(bool yes=true) { show_pid=yes; }
-   void ShowTime(bool yes=true) { show_time=yes; }
-   void ShowContext(bool yes=true) { show_context=yes; }
-   void ShowNothing() { show_pid=show_time=show_context=false; }
-
    static void Cleanup();
+
+   const char *ResPrefix() const { return "log"; }
+   const char *ResClosure() const { return name; }
+   void Reconfig(const char *);
 };
 
+#define debug(a) do { if(Log::global) Log::global->Format a; } while(0)
+
 #endif // LOG_H
