@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2020 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,9 +43,12 @@
 # include <sys/time.h>
 #endif
 
-#if LIBIDN
-# include <idna.h>
-# include <tld.h>
+#if LIBIDN2
+# include <idn2.h>
+#endif
+
+#ifdef HAVE_DLFCN_H
+# include <dlfcn.h>
 #endif
 
 CDECL_BEGIN
@@ -57,19 +60,19 @@ CDECL_END
 #include "SignalHook.h"
 #include "url.h"
 #include "ResMgr.h"
+#include "log.h"
 #include <mbswidth.h>
-#include "strftime.h"
 
 const char *dir_file(const char *dir,const char *file)
 {
    if(dir==0 || dir[0]==0)
       return file?file:dir;
-   if(file && file[0]=='.' && file[1]=='/')
-      file+=2;
    if(file==0 || file[0]==0)
       return dir;
    if(file[0]=='/')
       return file;
+   if(file[0]=='.' && file[1]=='/')
+      file+=2;
 
    xstring& buf=xstring::get_tmp();
    size_t len=strlen(dir);
@@ -102,8 +105,8 @@ const char *url_file(const char *url,const char *file)
       u.path.set(file);
    else
       u.path.set(dir_file(u.path,file));
-   buf.set_allocated(u.Combine());
-   return buf;
+   buf.truncate();
+   return u.CombineTo(buf);
 }
 
 const char *output_file_name(const char *src,const char *dst,bool dst_local,
@@ -176,7 +179,7 @@ const char *expand_home_relative(const char *s)
 
    if(s[1]==0 || s[1]=='/')
    {
-      home=getenv("HOME");
+      home=get_home();
    }
    else
    {
@@ -225,6 +228,8 @@ int   create_directories(char *path)
 	       return(-1);
 	    }
 	 }
+	 else
+	    debug((9,"mkdir(%s): ok\n",path));
       }
       if(sl)
 	 *sl++='/';
@@ -276,17 +281,37 @@ int fd_width(int fd)
 
 char *xgetcwd()
 {
+   char *cwd=getcwd(0,0); // glibc extension
+   if(cwd) {
+      xmalloc_register_block(cwd);
+      return cwd;
+   }
+
    int size=256;
+   cwd=(char*)xmalloc(size);
    for(;;)
    {
-      char *cwd=getcwd(0,size);
-      if(cwd)
-      {
-	 xmalloc_register_block(cwd);
+      if(getcwd(cwd,size))
 	 return cwd;
-      }
       if(errno!=ERANGE)
-	 return 0;
+	 return strcpy(cwd,".");
+      cwd=(char*)xrealloc(cwd,size*=2);
+   }
+}
+
+void xgetcwd_to(xstring& s)
+{
+   int size=256;
+   for(;;) {
+      s.get_space(size);
+      if(getcwd(s.get_non_const(),size)) {
+	 s.set_length(strlen(s.get()));
+	 return;
+      }
+      if(errno!=ERANGE) {
+	 s.set(".");
+	 return;
+      }
       size*=2;
    }
 }
@@ -560,9 +585,9 @@ bool re_match(const char *line,const char *a,int flags)
    return res;
 }
 
-char *Subst(const char *txt, const subst_t *s)
+xstring& SubstTo(xstring& buf,const char *txt, const subst_t *s)
 {
-   xstring buf("");
+   buf.nset("",0);
 
    char str[3];
    bool last_subst_empty=true;
@@ -622,7 +647,7 @@ char *Subst(const char *txt, const subst_t *s)
 
       buf.append(to_add);
    }
-   return(buf.borrow());
+   return(buf);
 }
 
 void xgettimeofday(time_t *sec, int *usec)
@@ -773,6 +798,9 @@ bool temporary_network_error(int err)
 #endif
 #ifdef ECONNABORTED
    case(ECONNABORTED):
+#endif
+#ifdef EADDRNOTAVAIL
+   case(EADDRNOTAVAIL):
 #endif
       return true;
    }
@@ -932,31 +960,44 @@ const xstring& shell_encode(const char *string,int len)
    return (result);
 }
 
-void remove_tags(char *buf)
+int remove_tags(char *buf)
 {
    int len=strlen(buf);
-   for(;;)
+   int less = -1;
+   for(int i = 0; i < len;i++)
    {
-      char *less=strchr(buf,'<');
-      char *amp=strstr(buf,"&nbsp;");
-      if(!less && !amp)
-	 break;
-      if(amp && (!less || amp<less))
-      {
-	 amp[0]=' ';
-	 memmove(amp+1,amp+6,len-(amp+6-buf)+1);
-	 len-=amp+6-buf;
-	 buf=amp+1;
-	 continue;
+      if(strcmp(buf + i, "&nbsp;") == 0){
+        for(int j = 0; j < 6; j++)buf[i + j] = 0;
+        buf[i] = ' ';
+        i += 6;
+        i--;
+        continue;
+      }
+      if(buf[i] == '<'){
+        less = i;
+        continue;
+
+      }
+      if(buf[i] == '>'){
+        if(less != -1){
+          for(int j = less; j <= i; j++)buf[j] = 0;
+          less = -1;
+        }
+
       }
-      char *more=strchr(less+1,'>');
-      if(!more)
-	 break;
-      memmove(less,more+1,len-(more+1-buf)+1);
-      len-=more+1-buf;
-      buf=less;
    }
+   int zero = 0;
+   for(int i = 0; i < len;i++)
+   {
+     while(zero < i && buf[zero] != 0)zero++;
+     if(buf[i] != 0 and zero != i){
+      buf[zero] = buf[i];
+      buf[i] = 0;
+     }
+   }
+   return ++zero;
 }
+
 void rtrim(char *s)
 {
    int len=strlen(s);
@@ -1012,24 +1053,26 @@ const char *xhuman(long long n)
 
 const char *xidna_to_ascii(const char *name)
 {
-#if LIBIDN
+#if LIBIDN2
    if(!name)
       return 0;
-   char *name_ace_tmp=0;
-   if(idna_to_ascii_lz(name,&name_ace_tmp,0)==IDNA_SUCCESS)
-      return xstring::get_tmp().set_allocated(name_ace_tmp);
-#endif//LIBIDN
+   static xstring_c name_ace_tmp;
+   name_ace_tmp.unset();
+   if(idn2_to_ascii_lz(name,name_ace_tmp.buf_ptr(),0)==IDN2_OK) {
+      xmalloc_register_block((void*)name_ace_tmp.get());
+      return name_ace_tmp;
+   }
+#endif//LIBIDN2
    return name;
 }
 bool xtld_name_ok(const char *name)
 {
-#if LIBIDN
+#if LIBIDN2
    if(mbswidth(name,MBSW_REJECT_INVALID|MBSW_REJECT_UNPRINTABLE)<=0)
       return false;
-   size_t err_pos;
-   if(tld_check_lz(name,&err_pos,NULL)==TLD_SUCCESS)
+   if(idn2_lookup_ul(name,NULL,0)==IDN2_OK)
       return true;
-#endif//LIBIDN
+#endif//LIBIDN2
    return false;
 }
 
@@ -1047,3 +1090,23 @@ bool is_ipv6_address(const char *s)
    return false;
 #endif
 }
+
+int lftp_fallocate(int fd,off_t sz)
+{
+#if defined(HAVE_FALLOCATE)
+   return fallocate(fd,0,0,sz);
+#elif defined(HAVE_POSIX_FALLOCATE)
+   return posix_fallocate(fd,0,sz);
+#else
+   errno=ENOSYS;
+   return -1;
+#endif
+}
+
+void call_dynamic_hook(const char *name) {
+#if defined(HAVE_DLOPEN) && defined(RTLD_DEFAULT)
+   typedef void (*func)();
+   func f=(func)dlsym(RTLD_DEFAULT,name);
+   if(f) f();
+#endif
+}
