@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,10 +34,16 @@ public:
 
    xstring_c orig_url;
 
+   ParsedURL() {}
    ParsedURL(const char *url,bool proto_required=false,bool use_rfc1738=true);
    void parse(const char *url,bool proto_required=false,bool use_rfc1738=true);
    ~ParsedURL();
 
+   xstring& CombineTo(xstring &buf,const char *home=0,bool use_rfc1738=true) const;
+   const char *CombineTo(xstring_c &buf,const char *home=0,bool use_rfc1738=true) const {
+      xstring tmp; tmp.move_here(buf);
+      return buf.move_here(CombineTo(tmp,home,use_rfc1738));
+   }
    // returns allocated memory
    char *Combine(const char *home=0,bool use_rfc1738=true);
 };
@@ -66,10 +72,10 @@ public:
    char *Combine();
 
    // encode unsafe chars as %XY
-   static xstring& encode(const char *s,int len,const char *unsafe,unsigned flags=0);
-   static xstring& encode(const xstring &s,const char *unsafe,unsigned flags=0) { return encode(s,s.length(),unsafe,flags); }
-   static xstring& encode(const char *s,const char *unsafe,unsigned flags=0) { return encode(s,strlen(s),unsafe,flags); }
-   static xstring& decode(const char *) __attribute__((warn_unused_result));
+   static const xstring& encode(const char *s,int len,const char *unsafe,unsigned flags=0);
+   static const xstring& encode(const xstring &s,const char *unsafe,unsigned flags=0) { return encode(s,s.length(),unsafe,flags); }
+   static const xstring& encode(const char *s,const char *unsafe,unsigned flags=0) { return encode(s,strlen(s),unsafe,flags); }
+   static const xstring& decode(const char *) __attribute__((warn_unused_result));
 
    static bool is_url(const char *p);
    static int path_index(const char *p);
